clear; close all; clc;
% this simple program draws a Bifurcation diagram for one-D differential
% equations, xDot = f(x,r), where x and r are both real numbers.

% range of r values at which we determine equilibria and their stability
rList = linspace(-1,1,40)';

numRandomSeeds = 8;

% for each of these r values, we determine all the equilibria and their
% stability
for i = 1:length(rList)
    rNow = rList(i);
    for j = 1:numRandomSeeds 
        
        % we re-do fsolve from a few random initial seeds so that we can
        % discover all (most) of the equilibria
        xInitialSeed = 2*randn; % normally distributed random number
        options = []; 
        xStar = fsolve(@fRHS,xInitialSeed,options,rNow);
        
        % checking if xStar is really a solution by checking if f(xStar) =
        % 0, or at least very small, and then doing 'linear stability
        % analysis'
        if abs(fRHS(xStar,rNow))<1e-6
            figure(1); hold on;
            if DfDx(xStar,rNow)<0 % negative slope => stable
                plot(rNow,xStar,'bo');
            elseif DfDx(xStar,rNow)>0
                plot(rNow,xStar,'ro'); % postive slope => unstable
            else
                plot(rNow,xStar,'ko'); % zero slope => indeterminate from linear stability analysis
            end
        end
    end
end 

xlabel('r'); ylabel('xStar (Equlibria)');
xlim([min(rList) max(rList)]);